/*!
// @file		EmaiObject.h
//
// @brief		Basic library objects
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIOBJECT_H__
#define __EMAIOBJECT_H__

#include <EmaiTypes.h>
#include <EmaiErrors.h>

#ifdef __cplusplus
	extern "C" {
#endif

/*!
// @brief	Base class for all Emai objects such as EmaiMessage, EmaiPop3Session,
//				etc. All library objects can be passed to the functions, which accept
//				EmaiObject as parameter.
*/
typedef void*									EmaiObject;

/*!
// @brief	Increments reference count value of the object by 1.
//
// @param	inObject - object which reference count value should be changed
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiObjectAddRef(
		EmaiObject inObject);

/*!
// @brief	Decreases reference count value of the object by 1. If reference
//				count become zero the object is destroyed and memory occupied by the
//				object is freed
//				This function returns EmaiErrIncorrectParameter error if caller trys
//				to destroy (set reference count to zero) message part objects which
//				are owned by message object.
//
// @param	inObject - object which reference count value should be changed
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiObjectRelease(
		EmaiObject inObject);


/*!
// @brief	Returns reference count value of the object.
//
// @param	inObject - object to be examined
// @param	outRefCount - on successful function exit contains reference count
//				value of the object
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiObjectGetRefCount(
		EmaiObject inObject,
		EmaiUint32* outRefCount);

/*!
// @brief	Allows to save user data to library object.
//
// @param	inObject - object to be modified
// @param	inKey - key used to identify the data
// @param	inData - pointer to data to be copied into object. If inData is NULL
//				the function will remove data from the object, inDataSize value
//				is ignored in this case
// @param	inDataSize - size of the data in bytes
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiObjectSetUserData(
		EmaiObject inObject,
		EmaiUint32 inKey,
		const void* inData,
		EmaiUint32 inDataSize);

/*!
// @brief	Allows to retrieve user data from library object stored in object
//				earlier using EmaiObjectSetUserData.
//
// @param	inObject - object to be examined
// @param	inKey - key used to identify the data
// @param	outData - pointer to memory where retrieved data should be stored.
//				If NULL ioDataSize contains size of the data stored under key inKey
// @param	ioDataSize - size of the outData block in bytes, on exit contains
//				size of the data copied in outData (if non-NULL) or size of the data
//				(if outData is NULL)
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiObjectGetUserData(
		EmaiObject inObject,
		EmaiUint32 inKey,
		void* outData,					/*!< can be NULL*/
		EmaiUint32* ioDataSize);

/*!
// @brief	Registers library.
//
// @param	inLicenseCode - license code
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiRegisterLibrary(
		EmaiConstUniCharPtr inLicenseCode);

#ifdef __cplusplus
	}
#endif

#endif /*__EMAIOBJECT_H__*/
